/* (C) 2022 YiRing, Inc. */
package com.yiring.app.domain.location;

import com.yiring.common.annotation.FieldMapping;
import com.yiring.common.domain.BasicEntity;
import java.io.Serial;
import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;
import javax.persistence.*;
import lombok.*;
import lombok.experimental.FieldDefaults;
import lombok.experimental.FieldNameConstants;
import lombok.experimental.SuperBuilder;
import org.hibernate.annotations.Comment;
import org.hibernate.annotations.Type;
import org.locationtech.jts.geom.Geometry;

/**
 * 围栏
 * 引用: 定位平台接口规范V3.0.1 #7.1
 *
 * @author Jim
 * @version 0.1
 * 2022/4/7 16:26
 */

@Getter
@Setter
@ToString
@SuperBuilder(toBuilder = true)
@NoArgsConstructor
@AllArgsConstructor
@FieldNameConstants
@FieldDefaults(level = AccessLevel.PRIVATE)
@Entity
@Table(name = "BS_LOCATION_FENCE", indexes = { @Index(columnList = "linkId"), @Index(columnList = "mode") })
@Comment("围栏")
public class LocationFence extends BasicEntity implements Serializable {

    @Serial
    private static final long serialVersionUID = 4155868702188991300L;

    @Comment("地图 ID")
    Long areaId;

    @FieldMapping("entityTypes")
    @Comment("围栏模型")
    @Enumerated(EnumType.STRING)
    Mode mode;

    @Comment("空间信息")
    @Type(type = "jts_geometry")
    @Column(columnDefinition = "geometry")
    Geometry geometry;

    @Comment("超时时间（秒）")
    Integer overtime;

    @Comment("消抖时间（秒）")
    Integer threshold;

    @Comment("信标集合")
    @Builder.Default
    @ManyToMany(mappedBy = "fences")
    @ToString.Exclude
    Set<LocationBeacon> beacons = new HashSet<>(0);

    @Comment("围栏规则集合")
    @Builder.Default
    @OneToMany(mappedBy = "fence")
    @ToString.Exclude
    Set<LocationFenceRule> rules = new HashSet<>(0);

    @SuppressWarnings({ "unused" })
    public enum Mode {
        NORMAL("常规区域"),
        DANGER("危险源区域");

        @Getter
        final String text;

        Mode(String text) {
            this.text = text;
        }
    }
}
