/* (C) 2022 YiRing, Inc. */
package com.yiring.app.vo.user;

import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.yiring.auth.domain.user.User;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import java.io.Serial;
import java.io.Serializable;
import java.time.LocalDateTime;
import java.util.List;
import java.util.stream.Collectors;
import lombok.*;
import lombok.experimental.FieldDefaults;

/**
 * 用户信息
 * @author ifzm
 * 2022/03/03 10:35
 **/

@ApiModel("UserVo")
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
@FieldDefaults(level = AccessLevel.PRIVATE)
public class UserVo implements Serializable {

    @Serial
    private static final long serialVersionUID = -2184378273450466835L;

    @JsonSerialize(using = ToStringSerializer.class)
    @ApiModelProperty(value = "主键", example = "1")
    Long id;

    @ApiModelProperty(value = "真实姓名", example = "超级用户")
    String realName;

    @ApiModelProperty(value = "工号", example = "8888888")
    String uuid;

    @ApiModelProperty(value = "类型", example = "员工")
    User.Type type;

    @ApiModelProperty(value = "部门", example = "系统管理员")
    String department;

    @ApiModelProperty(value = "手机号", example = "13012345678")
    String mobile;

    @ApiModelProperty(value = "职位", example = "系统管理员")
    String post;

    @ApiModelProperty(value = "标签编号", example = "BTT88888888")
    String code;

    @ApiModelProperty(value = "用户名", example = "admin")
    String username;

    @ApiModelProperty(value = "邮箱", example = "developer@yiring.com")
    String email;

    @ApiModelProperty(value = "头像", example = "https://s1.ax1x.com/2022/03/30/qggJH0.jpg")
    String avatar;

    @ApiModelProperty(value = "是否启用", example = "true")
    Boolean enabled;

    @ApiModelProperty(value = "是否删除", example = "false")
    Boolean deleted;

    @ApiModelProperty(value = "最后登录IP地址", example = "127.0.0.1")
    String lastLoginIp;

    @ApiModelProperty(value = "最后登录时间", example = "2022-10-24 10:24:00")
    LocalDateTime lastLoginTime;

    @ApiModelProperty(value = "创建时间", example = "2022-01-01 00:00:00")
    LocalDateTime createTime;

    public static UserVo transform(User user) {
        return UserVo
            .builder()
            .id(user.getId())
            .realName(user.getRealName())
            .username(user.getUsername())
            .mobile(user.getMobile())
            .email(user.getEmail())
            .enabled(user.getEnabled())
            .deleted(user.getDeleted())
            .lastLoginIp(user.getLastLoginIp())
            .lastLoginTime(user.getLastLoginTime())
            .createTime(user.getCreateTime())
            .build();
    }

    public static List<UserVo> transforms(List<User> users) {
        return users
            .stream()
            .map(user ->
                UserVo
                    .builder()
                    .id(user.getId())
                    .realName(user.getRealName())
                    .username(user.getUsername())
                    .mobile(user.getMobile())
                    .email(user.getEmail())
                    .avatar(user.getAvatar())
                    .enabled(user.getEnabled())
                    .deleted(user.getDeleted())
                    .lastLoginIp(user.getLastLoginIp())
                    .lastLoginTime(user.getLastLoginTime())
                    .createTime(user.getCreateTime())
                    .build()
            )
            .collect(Collectors.toList());
    }

    public UserVo(
        String realName,
        String uuid,
        User.Type type,
        String department,
        String mobile,
        String post,
        String code
    ) {
        this.realName = realName;
        this.uuid = uuid;
        this.type = type;
        this.department = department;
        this.mobile = mobile;
        this.post = post;
        this.code = code;
    }

    public static UserVo transformUserInfo(User user) {
        return UserVo
            .builder()
            .id(user.getId())
            .realName(user.getRealName())
            .username(user.getUsername())
            .mobile(user.getMobile())
            .email(user.getEmail())
            .avatar(user.getAvatar())
            .enabled(user.getEnabled())
            .deleted(user.getDeleted())
            .lastLoginIp(user.getLastLoginIp())
            .lastLoginTime(user.getLastLoginTime())
            .createTime(user.getCreateTime())
            .build();
    }
}
